#!/usr/bin/python
# -*- coding: utf-8 -*-

import unittest

import numpy as np

from pypitch.pypitch import Analyzer
from pypitch.pypitch import Tone


class AnalyzerTest(unittest.TestCase):

    def setUp(self):
        self.rate = 44100
        self.instr = np.array([[1, 1],
                               [1, 2]], dtype=np.float32)
        self.analyzer = Analyzer(self.rate)

    def test_init(self):
        Analyzer(self.rate)

    def test_peak(self):
        self.analyzer.input(self.instr)
        self.analyzer.process()
        cpeak = self.analyzer.getPeak()
        peak = 6.020599913279624

        self.assertEquals(cpeak, peak)

    def test_getFormants(self):
        self.analyzer.input(self.instr)
        self.analyzer.process()
        cformants = self.analyzer.getFormants()
        formants = [None] * 3

        self.assertEquals(cformants, formants)

    #def test_findTone(self):
    #    self.analyzer.input(self.instr)
    #    self.analyzer.process()
    #    ctone = self.analyzer.findTone()

    #    self.assertIsInstance(ctone, Tone)



#!/usr/bin/env python


# from distutils.core import setup
# from distutils.extension import Extension
# from distutils.sysconfig import get_python_lib
from setuptools import setup
from setuptools import Extension
import os
import sys

from Cython.Build import cythonize
# from Cython.Distutils import build_ext


readme_filepath = os.path.join(os.path.dirname(__file__), "README.md")
try:
    import pypandoc
    long_description = pypandoc.convert(readme_filepath, 'rst')
except ImportError:
    long_description = open(readme_filepath).read()


# sources
ext_sources = [
    'pypitch/_pypitch.pyx',
    'pypitch/pitch.cpp'
]

# extension
extra_compile_args_pitch = list()
extra_link_args_pitch = list()
if sys.platform.startswith("darwin"):
    # hack: with mac os version 10.13.6 (high sierra), xcode 10 requires the use of the libc++ standard library
    extra_compile_args_pitch.extend(["-stdlib=libc++", "-mmacosx-version-min=10.9"])
    extra_link_args_pitch.extend(["-stdlib=libc++", "-mmacosx-version-min=10.9"])

ext = Extension(
    name='pypitch.pypitch',
    sources=ext_sources,
    language='c++',
    extra_compile_args=extra_compile_args_pitch,
    extra_link_args=extra_link_args_pitch,
)

# setup
setup(
    name='pypitch',
    version='1.1',
    description='PyPitch analyses audio streams for pitch',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='FoFiX team',
    author_email='fofix@perdu.fr',
    license='GPLv2+',
    url='https://fofix.github.io',
    project_urls={
        'Documentation': 'https://pypitch.readthedocs.io',
        'Source Code': 'https://github.com/fofix/python-pypitch',
        'Bug Tracker': 'https://github.com/fofix/python-pypitch/issues',
    },
    packages=['pypitch'],
    #package_data={'pypitch': ['*.dll']},
    zip_safe=False,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        "Programming Language :: C++",
        "Programming Language :: Cython",
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Topic :: Multimedia',
        'Topic :: Multimedia :: Sound/Audio',
        'Topic :: Software Development :: Libraries',
    ],
    keywords='pitch audio',
    ext_modules=cythonize(ext, compiler_directives={'language_level': sys.version_info[0]}),
    setup_requires=['cython', 'pytest-runner'],
    install_requires=[
        'Cython >= 0.27',
    ],
    test_suite="tests",
    tests_require=['pytest<5', 'numpy<1.17'],
    extras_require={
        'tests': ['pytest<5', 'numpy<1.17'],
        'docs': ['sphinx', 'sphinx_rtd_theme'],
    },
)



# pypitch - analyze audio streams for pitch
# Copyright (C) 2008-2009, 2015 John Stumpo
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

__version__ = '1.0'

from libcpp.vector cimport vector

cdef extern from "pitch.hpp" nogil:
    cdef cppclass _Tone 'Tone':
        double freq
        double db
        double stabledb
        double* harmonics
        int age
    cdef cppclass _Analyzer 'Analyzer':
        _Analyzer(double)
        void input[T](T, T)
        void process()
        double getPeak()
        vector[double] getFormants()
        const _Tone* findTone(double, double)


class Tone:
    def __init__(self, freq, db, stabledb, harmonics, age):
        self.freq = freq
        self.db = db
        self.stabledb = stabledb
        self.harmonics = harmonics
        self.age = age

cdef object PyTone_FromTone(const _Tone* t):
    cdef int i
    if t == NULL:
        return None
    harmonics = []
    for 0 <= i < 8:
        harmonics.append(t.harmonics[i])
    return Tone(t.freq, t.db, t.stabledb, harmonics, t.age)

cdef class Analyzer:
    cdef _Analyzer* _this
    def __cinit__(self, double rate):
        self._this = new _Analyzer(rate)
    def __dealloc__(self):
        del self._this
    def input(self, instr):
        cdef float* begin
        if not isinstance(instr, str):
            instr = instr.tostring()  # assume it was a numpy array
        begin = <float*><char*>instr
        self._this.input(begin, begin + (len(instr) / sizeof(float)))
    def process(self):
        self._this.process()
    def getPeak(self):
        return self._this.getPeak()
    def getFormants(self):
        cdef vector[double] v
        cdef unsigned int i
        cdef double cur
        v = self._this.getFormants()
        formants = []
        for 0 <= i < v.size():
            cur = v[i]
            if cur == 0.0:
                formants.append(None)
            else:
                formants.append(cur)
        return formants
    def findTone(self, double minfreq=65.0, double maxfreq=1000.0):
        return PyTone_FromTone(self._this.findTone(minfreq, maxfreq))
